@@ -2,8 +2,8 @@
 # -*- coding: utf-8 -*-
 
 from collections import defaultdict
-from datetime import datetime
-from methodtools import lru_cache
+from datetime import datetime, timedelta
+import math
 import numpy as np
 import pandas as pd
 from covsirphy.util.error import deprecate
@@ -38,6 +38,7 @@ class Term(object):
     STR_COLUMNS = [DATE, *AREA_COLUMNS]
     COLUMNS = [*STR_COLUMNS, C, CI, F, R]
     NLOC_COLUMNS = [DATE, C, CI, F, R]
+    SUB_COLUMNS = [DATE, C, CI, F, R, S]
     VALUE_COLUMNS = [C, CI, F, R]
     FIG_COLUMNS = [CI, F, R, FR, V, E, W]
     # Date format: 22Jan2020 etc.
@@ -56,12 +57,17 @@ class Term(object):
     # Phase name
     SUFFIX_DICT = defaultdict(lambda: "th")
     SUFFIX_DICT.update({1: "st", 2: "nd", 3: "rd"})
+    # Summary of phases
     TENSE = "Type"
     PAST = "Past"
     FUTURE = "Future"
     INITIAL = "Initial"
     ODE = "ODE"
     RT = "Rt"
+    RMSLE = "RMSLE"
+    TRIALS = "Trials"
+    RUNTIME = "Runtime"
+    EST_COLS = [RMSLE, TRIALS, RUNTIME]
     # Scenario analysis
     PHASE = "Phase"
     SERIES = "Scenario"
@@ -69,17 +75,16 @@ class Term(object):
     # Flag
     UNKNOWN = "-"
 
-    @lru_cache(maxsize=None)
     @classmethod
     def num2str(cls, num):
         """
         Convert numbers to 1st, 2nd etc.
 
         Args:
-        @num (int): number
+            num (int): number
 
         Returns:
-            (str)
+            str
         """
         if not isinstance(num, int):
             raise TypeError("@num must be an integer.")
@@ -87,6 +92,24 @@ class Term(object):
         suffix = "th" if q == 1 else cls.SUFFIX_DICT[mod]
         return f"{num}{suffix}"
 
+    @staticmethod
+    def str2num(string, name="phase names"):
+        """
+        Convert 1st to 1 and so on.
+
+        Args:
+            string (str): like 1st, 2nd, 3rd,...
+            name (str): name of the string
+
+        Returns:
+            int
+        """
+        try:
+            return int(string[:-2])
+        except ValueError:
+            raise ValueError(
+                f"Examples of {name} are 0th, 1st, 2nd..., but {string} was applied.")
+
     @staticmethod
     def negative_exp(x, a, b):
         """
@@ -96,28 +119,11 @@ class Term(object):
             x (float): x values
             a (float): the first parameters of the function
             b (float): the second parameters of the function
-        """
-        return a * np.exp(-b * x)
-
-    @lru_cache(maxsize=None)
-    @classmethod
-    def date_obj(cls, date_str):
-        """
-        Convert a string to a datetime object.
-
-        Args:
-            date_str (str or None): date, like 22Jan2020
 
         Returns:
-            (datetime.datetime or None): datetime object
-
-        Notes:
-            If @date_str is None, None will be returned.
+            float
         """
-        if date_str is None:
-            return None
-        obj = datetime.strptime(date_str, cls.DATE_FORMAT)
-        return obj
+        return a * np.exp(-b * x)
 
     @staticmethod
     def flatten(nested_list, unique=True):
@@ -129,7 +135,7 @@ class Term(object):
             unique (bool): if True, only unique values will remain
 
         Returns:
-            (list[object])
+            list[object]
         """
         flattened = sum(nested_list, list())
         if unique:
@@ -157,29 +163,37 @@ class Term(object):
         if columns is None:
             return df
         if not set(columns).issubset(set(df.columns)):
-            cols_str = ', '.join(
+            cols_str = ", ".join(
                 [col for col in columns if col not in df.columns]
             )
+            included = ", ".join(df.columns.tolist())
+            s1 = f"Expected columns were not included in {name} with {included}."
             raise KeyError(
-                f"Expected columns were not included in {name}. {cols_str} must be included."
+                f"{s1} However, {cols_str} must be included."
             )
         return df
 
     @staticmethod
-    def ensure_natural_int(target, name="number", include_zero=False):
+    def ensure_natural_int(target, name="number", include_zero=False, none_ok=False):
         """
         Ensure a natural (non-negative) number.
-        If the value is a natural number and the type was float or string,
-        it will be converted to an integer.
 
         Args:
-            target (int or float or str): value to ensure
+            target (int or float or str or None): value to ensure
             name (str): argument name of the value
             include_zero (bool): include 0 or not
+            none_ok (bool): None value can be applied or not.
 
         Returns:
-            (int): as-is the target
+            int: as-is the target
+
+        Notes:
+            When @target is None and @none_ok is True, None will be returned.
+            If the value is a natural number and the type was float or string,
+            it will be converted to an integer.
         """
+        if target is None and none_ok:
+            return None
         s = f"@{name} must be a natural number, but {target} was applied"
         try:
             number = int(target)
@@ -192,6 +206,41 @@ class Term(object):
             raise ValueError(f"{s}. This value is under {min_value}")
         return number
 
+    @classmethod
+    def ensure_tau(cls, tau):
+        """
+        Ensure that the value can be used as tau value [min].
+
+        Args:
+            tau (int or None): value to use
+
+        Returns:
+            int or None: as-is
+        """
+        if tau is None:
+            return None
+        tau = cls.ensure_natural_int(tau, name="tau")
+        if tau in set(cls.divisors(1440)):
+            return tau
+        raise ValueError(
+            f"@tau must be a divisor of 1440 [min], but {tau} was applied."
+        )
+
+    @classmethod
+    def ensure_population(cls, population):
+        """
+        Ensure that the population value is valid.
+
+        Args:
+            population (int or float or str): population value
+
+        Returns:
+            int: as-is
+        """
+        return cls.ensure_natural_int(
+            population, name="population", include_zero=False, none_ok=False
+        )
+
     @staticmethod
     def ensure_float(target, name="value"):
         """
@@ -204,7 +253,7 @@ class Term(object):
             name (str): argument name of the value
 
         Returns:
-            (float): as-is the target
+            float: as-is the target
         """
         s = f"@{name} must be a float value, but {target} was applied"
         try:
@@ -223,7 +272,7 @@ class Term(object):
             name (str): argument name of the string
 
         Returns:
-            (str): as-is the target
+            str: as-is the target
         """
         try:
             cls.date_obj(target)
@@ -244,7 +293,7 @@ class Term(object):
             name (str): argument name of the target
 
         Returns:
-            (int): as-is the target
+            int: as-is the target
         """
         s = f"@{name} must be an sub class of {type(parent)}, but {type(target)} was applied."
         if not issubclass(target, parent):
@@ -262,14 +311,13 @@ class Term(object):
             name (str): argument name of the target
 
         Returns:
-            (instance): as-is target
+            object: as-is target
         """
         s = f"@{name} must be an instance of {type(class_obj)}, but {type(target)} was applied."
         if not isinstance(target, class_obj):
             raise TypeError(s)
         return target
 
-    @lru_cache(maxsize=None)
     @classmethod
     def divisors(cls, value):
         """
@@ -279,17 +327,15 @@ class Term(object):
             value (int): target value
 
         Returns:
-            (list[int]): the list of divisors
+            list[int]: the list of divisors
         """
         value = cls.ensure_natural_int(value)
-        divisors = [
+        return [
             i for i in range(1, value + 1) if value % i == 0
         ]
-        return divisors
 
-    @lru_cache(maxsize=None)
     @classmethod
-    def to_date_obj(cls, date_str=None, default=None):
+    def date_obj(cls, date_str=None, default=None):
         """
         Convert a string to a datatime object.
 
@@ -298,18 +344,97 @@ class Term(object):
             default (datetime.datetime or None, optional): default value to return
 
         Returns:
-            (datetime.datetime or None)
+            datetime.datetime or None
 
         Notes:
             If @date_str is None, returns @default value
         """
         if date_str is None:
             return default
-        date_obj = datetime.strptime(date_str, cls.DATE_FORMAT)
-        return date_obj
+        return datetime.strptime(date_str, cls.DATE_FORMAT)
+
+    @classmethod
+    def date_change(cls, date_str, days=0):
+        """
+        Return @days days ago or @days days later.
+
+        Args:
+            date_str (str): today
+            days (int): (negative) days ago or (positive) days later
+
+        Returns:
+            str: the date
+        """
+        if not isinstance(days, int):
+            raise TypeError(
+                f"@days must be integer, but {type(days)} was applied.")
+        date = cls.date_obj(date_str) + timedelta(days=days)
+        return date.strftime(cls.DATE_FORMAT)
+
+    @classmethod
+    def tomorrow(cls, date_str):
+        """
+        Tomorrow of the date.
+
+        Args:
+            date_str (str): today
+
+        Returns:
+            str: tomorrow
+        """
+        return cls.date_change(date_str, days=1)
+
+    @classmethod
+    def yesterday(cls, date_str):
+        """
+        Yesterday of the date.
+
+        Args:
+            date_str (str): today
+
+        Returns:
+            str: yesterday
+        """
+        return cls.date_change(date_str, days=-1)
+
+    @classmethod
+    def steps(cls, start_date, end_date, tau):
+        """
+        Return the number of days (round up).
+
+        Args:
+            start_date (str): start date, like 01Jan2020
+            end_date (str): end date, like 01Jan2020
+            tau (int): tau value [min]
+        """
+        sta = cls.date_obj(start_date)
+        end = cls.date_obj(end_date)
+        tau = cls.ensure_tau(tau)
+        return math.ceil((end - sta) / timedelta(minutes=tau))
+
+    @classmethod
+    def ensure_date_order(cls, previous_date, following_date, name="following_date"):
+        """
+        Ensure that the order of dates.
+
+        Args:
+            previous_date (str): previous date
+            following_date (str): following date
+            name (str): name of @following_date
+
+        Raises:
+            ValueError: @previous_date > @following_date
+        """
+        previous = cls.date_obj(previous_date)
+        following = cls.date_obj(following_date)
+        if previous <= following:
+            return None
+        raise ValueError(
+            f"@{name} must be the same as/over {previous_date}, but {following_date} was applied."
+        )
 
 
 class Word(Term):
-    @deprecate(old="Word()", new="Term()")
+    @ deprecate(old="Word()", new="Term()")
     def __init__(self):
         super().__init__()
