@@ -34,8 +34,8 @@ class SIRFV(ModelBase):
         "v": ModelBase.V
     }
     VARIABLES = list(VAR_DICT.values())
-    # Priorities of the variables when optimization
-    PRIORITIES = np.array([0, 10, 10, 2, 0])
+    # Weights of variables in parameter estimation error function
+    WEIGHTS = np.array([0, 10, 10, 2, 0])
     # Variables that increases monotonically
     VARS_INCLEASE = [ModelBase.R, ModelBase.F]
     # Example set of parameters and initial values
